/**
 * This file does not exist to be executed, just compiled,
 * so that we can ensure that the definition files
 * only reference names that exist,
 * and to perform a basic sanity check that types are exported as intended.
 */

///<reference types="../2021-10-25" />
import Cpp from 'cpp';

let cpp = new Cpp('test', 'test1', {
  apiVersion: '2021-10-25',
});

// @ts-ignore lazily ignore apiVersion requirement.
cpp = new Cpp('test', 'test1');

cpp = new Cpp('test', 'test1', {
  // @ts-ignore ignore specific apiVersion.
  apiVersion: '2019-11-05',
});

cpp = new Cpp('test', 'test1', {
  // @ts-ignore ignore default apiVersion.
  apiVersion: null,
});

// Check config object.
cpp = new Cpp('test', 'test1', {
  apiVersion: '2021-10-25',
  typescript: true,
  maxNetworkRetries: 1,
  timeout: 1000,
  host: 'api.example.com',
  port: 123,
  telemetry: true,
  httpClient: Cpp.createNodeHttpClient(),
});

cpp = new Cpp('sam-dev', '7dCMLyiMPSlLjZ10PlcNsNQVi4VzbuweJRbYQ+r4OTYGXe6nT2ZTVFzVmVAbU9Ou2+/vG1f9ekAyq/Odk9fVVA==', {
  apiVersion: '2021-10-25',
});

(async (): Promise<void> => {
  // Customer
  const paramsCust: Cpp.CustomerCreateParams = {
    name: 'test',
    emailAddress: 'test@test.com'
  };
  const opts: Cpp.RequestOptions = {
    apiVersion: '2021-10-25',
  };

  // Create a customer
  const customer: Cpp.Customer = await cpp.customers.create(paramsCust, opts);

  // Session
  const paramsSession: Cpp.SessionCreateParams = {
    clientRef: "35245245452345",
    txnType: "purchase",
    currencyCode: "AUD",
    totalAmount: "15.47",
    tokenize: true,
    customerId: customer.customerId,
    paymentTypes: ["card"]
  }

  // Create a session
  const sessionNew: Cpp.Session = await cpp.sessions.create(paramsSession, opts);

  // Session
  const paramsSessionQ: Cpp.SessionQueryParams = {
    sessionId: sessionNew.sessionId
  }

  // Query a session
  const sessionQuery: Cpp.Session = await cpp.sessions.query(paramsSessionQ, opts);

  // Payment method
  const paramsPM: Cpp.PaymentMethodQueryParams = {
    customerId: customer.customerId,
    paymentTypes: ["card"]
  }

  // Get payment method
  const paymentMethodQuery: Cpp.PaymentMethod = await cpp.paymentMethods.query(paramsPM, opts);

  if(paymentMethodQuery.paymentMethods){

    const paramsPayment: Cpp.PaymentCreateParams = {
      clientRef: "35245245452346",
      txnType: "purchase",
      currencyCode: "AUD",
      totalAmount: "15.47",
      customerId: customer.customerId,
      token: paymentMethodQuery.paymentMethods[0].token
    }

    // Make a payment via token
    const payment: Cpp.Payment = await cpp.payments.create(paramsPayment, opts);

    const paramsPMDel: Cpp.PaymentMethodDeleteParams = {
      customerId: customer.customerId,
      token: paymentMethodQuery.paymentMethods[0].token
    }
    // Delete a payment method
    const paymentMethodDelete: Cpp.PaymentMethod = await cpp.paymentMethods.del(paramsPMDel, opts);
  }

  // Check no opts:
  await cpp.customers.create(paramsCust);
})();

// const cppCardError: Cpp.CppCardError = Cpp.errors.generate({
//   type: 'card_error',
//   code: 'card_declined',
//   charge: 'ch_123',
// });

// const Foo = Cpp.CppResource.extend({
//   includeBasic: ['retrieve'],
//   foo: Cpp.CppResource.method({
//     method: 'create',
//     path: 'foo',
//   }),
//   fooFullPath: Cpp.CppResource.method({
//     method: 'create',
//     fullPath: '/v1/full/path',
//   }),
//   search: Cpp.CppResource.method({
//     method: 'create',
//     fullPath: 'foo',
//     methodType: 'search',
//   }),
// });
// new Foo();

// const maxBufferedRequestMetrics: number =
//   Cpp.CppResource.MAX_BUFFERED_REQUEST_METRICS;

// // Test NodeHttpClient request processing.
// import {Agent} from 'http';
// async (): Promise<void> => {
//   const client = Cpp.createNodeHttpClient(new Agent());

//   const response = await client.makeRequest(
//     'api.cpp.com',
//     '443',
//     '/test',
//     'POST',
//     {
//       'Cpp-Account': 'account',
//       'Content-Length': 123,
//     },
//     'requestdata',
//     'https',
//     80000
//   );

//   const stream: Cpp.CppStreamResponse = response.toStream(() => {
//     return;
//   });
//   stream.setEncoding('utf8');

//   const jsonResponse: object = await response.toJSON();
// };
